﻿using System;
using System.Reflection;

namespace JustinAngel.MetroBlog.Common.Helpers
{
    public static class ReflectionHelper
    {
        public static object GetProperty(this object obj, string propertyName)
        {
            return getPropInfo(obj, propertyName).GetValue(obj, null);
        }

        public static void SetProperty(this object obj, string propertyName, object propertyValue)
        {
            getPropInfo(obj, propertyName).SetValue(obj, propertyValue, null);
        }

        private static PropertyInfo getPropInfo(object obj, string propertyName)
        {
            return obj.GetType().GetProperty(propertyName, BindingFlags.Public | BindingFlags.Instance | BindingFlags.FlattenHierarchy);
        }

        public static void InitializeProperty(this object obj, string propertyName)
        {
            PropertyInfo propInfo = getPropInfo(obj, propertyName);
            if (propInfo != null)
            {
                propInfo.SetValue(obj, Activator.CreateInstance(propInfo.PropertyType), null);
            }

        }

        public static bool PropertyExists(this object obj, string propertyName)
        {
            return (getPropInfo(obj, propertyName) != null);
        }

        public static Type GetPropertyType(this object obj, string propertyName)
        {
            return getPropInfo(obj, propertyName).PropertyType;
        }

        public static object Invoke(this object obj, string MethodName, params object[] parameters)
        {
            return obj.GetType().GetMethod(MethodName).Invoke(obj, parameters);
        }

        public static object InvokeGeneric(this object obj, string MethodName, Type T)
        {
            return obj.GetType().GetMethod(MethodName).MakeGenericMethod(T).Invoke(obj, null);
        }
    }
}
