﻿using System.Linq;
using System.Web.Security;
using JustinAngel.MetroBlog.Common.Helpers;
using JustinAngel.MetroBlog.Services.Data;
using JustinAngel.MetroBlog.Services.Repository;
using Microsoft.Practices.Unity;

namespace JustinAngel.MetroBlog.Services.UserService
{
    public class UserService : IUserService
    {
        [Dependency]
        public IRepository<User> UserRepository { get; set; }

        [Dependency]
        public IRepository<BlogPost> BlogPostRepository { get; set; }

        public bool IsValid(string Username, string Password)
        {
            string passwordHash = FormsAuthentication.HashPasswordForStoringInConfigFile(Password, "sha1");
            return UserRepository.Get().Any(
                u => u.Username == Username
                  && u.PasswordHash == passwordHash);
        }

        public User GetUserByUserName(string username)
        {
            return UserRepository.Get().First(u => u.Username == username);
        }



        public User GetUserByUserNameOrDefaultUser(string username)
        {
            if (string.IsNullOrEmpty(username))
                return GetIsDefaultBlog();

            User ReturnValue = UserRepository.Get()
                                             .With(u => u.Categories)
                                             .With(u => u.Categories.Ignore().ChildCategories)
                                             .FirstOrDefault(u => u.Username == username);

            if (ReturnValue != null)
                return ReturnValue;
            else
                return GetIsDefaultBlog();
        }

        private User GetIsDefaultBlog()
        {
            return UserRepository.Get()
                .With(u => u.Categories)
                .With(u => u.Categories.Ignore().ChildCategories)
                .First(u => u.IsDefaultBlog.HasValue && u.IsDefaultBlog.Value);
        }
    }
}
