using System;
using System.Linq.Expressions;
using System.Reflection;
using System.Web.Mvc;
using System.Web.Security;
using JustinAngel.MetroBlog.Common.IoC;
using JustinAngel.MetroBlog.Services.Data;
using JustinAngel.MetroBlog.Services.UserService;

namespace JustinAngel.MetroBlog.Controllers
{
    public class LoginController : Controller
    {
        [Microsoft.Practices.Unity.Dependency]
        public IUserService UserService { get; set; }

        public LoginController()
        {
            IoC.BuildUp(this);
        }

        public ActionResult Load()
        {
            return View("Login");
        }

        public ActionResult Submit()
        {
            User userToValidate = new User();
            this.TryUpdateModel(userToValidate);

            if (UserService.IsValid(userToValidate.Username, userToValidate.Password))
            {
                if (Request.QueryString["ReturnUrl"] != null)
                {
                    FormsAuthentication.RedirectFromLoginPage(userToValidate.Username, false);
                    return null;
                }
                else
                {
                    FormsAuthentication.SetAuthCookie(userToValidate.Username, false);
                    return new RedirectResult("~/Management");
                }
            }
            else
            {
                ModelState.AddModelError(MvcHelper.GetPropertyName((User u)=> u.Password),  "Authentication Failed.");
                return View("Login", userToValidate);
            }
        }

    }

    public static class MvcHelper
    {
        public static string GetPropertyName<T>(Expression<Func<T, string>> property)
        {
            var propertyInfo = (property.Body as MemberExpression).Member as PropertyInfo;
            if (propertyInfo == null)
            {
                throw new ArgumentException("The lambda expression 'property' should point to a valid Property");
            }

            var propertyName = propertyInfo.Name;

            return propertyName;
        }

    }
}
